// Program prenasejici data ze ZX Spectra s UR-4 pres kartu FPC-024 s 8255 v ISA ...

program transfer;

uses	go32, crt;

const   PA1 = $1B0;
        PB1 = $1B1;
        PC1 = $1B2;
        PS1 = $1B3;
        PA2 = $1B4;
        PB2 = $1B5;
        PC2 = $1B6;
        PS2 = $1B7;
        TRACKLENGTH = 4096;

type    TTrack = array [0..TRACKLENGTH-1] of byte;

function InputDiskName (FileName: string): string;
var     s: string;
begin
        textcolor (2);
        gotoxy (40,1);
        write ('Diskname:                     ');
        textcolor (8);
        gotoxy (49,2);
        write ('(',FileName,')          ');
        textcolor (14);
        s := '';
        gotoxy (50,1);
        readln (s);
        InputDiskName := s;
end;

procedure PrintStatus (t,b,d: longint);
begin
        textcolor (14);
        gotoxy (1,2);
        write ('Track: ',t,'   ');
        gotoxy (1,3);
        write ('Byte:  ',b,'   ');
        gotoxy (1,4);
        write ('Discs: ',d,'   ');
end;

var     b: byte;
        key: char;
        BufferIndex: longint;
        TrackBuffer: TTrack;
        DataReaded: boolean;
        EndOfTransfer: boolean;
        WaitState: (nothing, data);     // na co se ceka
        output: file of TTrack;
        OutputFileName: string;
        CounterBytes: longint;
        CounterTracks: longint;
        CounterDiscs: longint;
begin
        // inicializace portu
        // port A - vstup (stejne neni propojen)
        // port B - vstup (ze ZX do PC)
        // port C (0 - 3) - vstup
        // port C (4 - 7) - vystup
        outportb (PS1, %10011010);
        outportb (PC1, 0);
        EndOfTransfer := false;
        BufferIndex := 0;
        CounterBytes := 0;
        CounterTracks := 0;
        CounterDiscs := 0;
        OutputFileName := 'image.trd';
        if (inportb (PC1) and 16) > 0 then
                WaitState := data
        else
                WaitState := nothing;

        // pred spustenim
        clrscr;
        gotoxy (1,25);
        textcolor (4);
        writeln ('Q = quit, N = new disk, S = port status');
        gotoxy (1,1);
        textcolor (2);
        writeln ('Ready: ');
        OutputFileName := InputDiskName (OutputFileName);
        assign (output, OutputFileName);
        rewrite (output);
        PrintStatus (CounterTracks, CounterBytes, CounterDiscs);

        repeat
                // zpracovani vstupu od uzivatele
                if keypressed then
                begin
                        key := readkey;
                        case key of
                                'q':
                                begin
                                        writeln ('QUIT - program interrupted by user');
                                        EndOfTransfer := true;
                                end;
                                'n':
                                begin
                                        close (output);
                                        OutputFileName := InputDiskName (OutputFileName);
                                        assign (output, OutputFileName);
                                        rewrite (output);
                                        BufferIndex := 0;
                                        CounterBytes := 0;
                                        CounterTracks := 0;
                                        inc (CounterDiscs);
				PrintStatus (CounterTracks, CounterBytes, CounterDiscs);
                                end;
                                's':
                                begin
                                        textcolor (15);
                                        gotoxy (1,2);
                                        writeln ('B: ',inportb (PB1),'       ');
                                        writeln ('C: ',inportb (PC1),'       ');
                                        writeln ('               ');
                                        delay (1000);
                                        PrintStatus (CounterTracks, CounterBytes, CounterDiscs);
                                end;
                        end;
                end;

                // precti data do promenne b
                DataReaded := false;
                case WaitState of
                        nothing:
                        begin
                                // cekam, dokud neni ridici bit nulovy, pak
                                // budu ocekavat jeho zmenu na 1, kdy budou
                                // data platna STROBE z 1 do 0
                                if (inportb (PC1) and %00010000) = 0 then
                                begin
                                        WaitState := data;
                                        outportb (PC1,%00000000); // ACK na 0
                                end;
                        end;
                        data:
                        begin
                                // cekam, dokud neni ridici bit 1, v tu chvili
                                // jsou data platna a prectu je, pak budu zas
                                // cekat dokud bit z 1 nespadne na 0
                                if (inportb (PC1) and 16) > 0 then // data platna
                                begin
                                        b := inportb (PB1);
                                        TrackBuffer [BufferIndex] := b;
                                        outportb (PC1,%00001111 ); // ACK
                                        DataReaded := true;
                                        WaitState := nothing;
                                        inc (CounterBytes);
                                        inc (BufferIndex);
                                        if BufferIndex > TRACKLENGTH - 1 then
                                        begin
                                                // - pridej k otevrenemu souboru
                                                BufferIndex := 0;
                                                inc (CounterTracks);
                                                write (output, TrackBuffer);
                                        end;
                                        PrintStatus (CounterTracks, CounterBytes, CounterDiscs);
                                end;
                        end;
                end;

        until EndOfTransfer;
        outportb (PS1, %10000000);      // vse jako vystup
        outportb (PA1, 0);
        outportb (PB1, 0);
        outportb (PC1, 0);
        close (output);
        textcolor (7);
        writeln ('O.K.');
end.
